@@ -3,7 +3,7 @@
  *
  *  binkd.c is a part of binkd project
  *
- *  Copyright (C) 1996-1998  Dima Maloff, 5047/13
+ *  Copyright (C) 1996-2003  Dima Maloff 5047/13 and others
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -12,10 +12,125 @@
  */
 
 /*
- * $Id: binkd.c,v 2.1 2001/01/15 22:04:52 gul Exp $
+ * $Id: binkd.c,v 2.30.2.7 2003/08/28 09:22:25 stas Exp $
  *
  * $Log: binkd.c,v $
- * Revision 2.1  2001/01/15 22:04:52  gul
+ * Revision 2.30.2.7  2003/08/28 09:22:25  stas
+ * Cosmetics
+ *
+ * Revision 2.30.2.6  2003/08/28 06:29:51  hbrew
+ * Update binkd9x for compability with binkd 0.9.6 commandline; put binkd9x.txt from current
+ *
+ * Revision 2.30.2.5  2003/08/27 12:59:35  stas
+ * Update usage(), optimize code
+ *
+ * Revision 2.30.2.4  2003/08/27 12:00:46  stas
+ * Fix binkd9x build
+ *
+ * Revision 2.30.2.3  2003/08/27 10:22:28  stas
+ * Make binkd 0.9.5 command line compatible with binkd 0.9.6
+ *
+ * Revision 2.30.2.2  2003/07/17 04:35:23  hbrew
+ * Fix "No prototype found for 'isdigit'" warning.
+ *
+ * Revision 2.30.2.1  2003/06/30 22:46:03  hbrew
+ * Print only binkd name (without path) in error messages
+ *
+ * Revision 2.30  2003/05/10 00:30:36  hbrew
+ * binkd9x: -u option now support '--all' service name (uninstall all services).
+ * Unneeded spaces cleanup.
+ *
+ * Revision 2.29  2003/05/04 08:45:30  gul
+ * Lock semaphores more safely for resolve and IP-addr print
+ *
+ * Revision 2.28  2003/05/01 09:55:01  gul
+ * Remove -crypt option, add global -r option (disable crypt).
+ *
+ * Revision 2.27  2003/03/31 21:48:59  gul
+ * Avoid infinite recursion
+ *
+ * Revision 2.26  2003/03/31 19:53:08  gul
+ * Close socket before exit
+ *
+ * Revision 2.25  2003/03/31 19:35:16  gul
+ * Clean semaphores usage
+ *
+ * Revision 2.24  2003/03/11 11:42:23  gul
+ * Use event semaphores for exit threads
+ *
+ * Revision 2.23  2003/03/11 00:04:25  gul
+ * Use patches for compile under MSDOS by MSC 6.0 with IBMTCPIP
+ *
+ * Revision 2.22  2003/03/10 10:39:23  gul
+ * New include file common.h
+ *
+ * Revision 2.21  2003/03/10 08:38:07  gul
+ * Make n_servers/n_clients changes thread-safe
+ *
+ * Revision 2.20  2003/03/05 13:21:50  gul
+ * Fix warnings
+ *
+ * Revision 2.19  2003/03/04 07:27:11  gul
+ * Put OS to log on start
+ *
+ * Revision 2.18  2003/03/03 09:43:07  gul
+ * Cosmetics changes
+ *
+ * Revision 2.17  2003/03/01 17:42:32  gul
+ * Added tzset() call to main() start
+ *
+ * Revision 2.16  2003/03/01 17:33:25  gul
+ * Clean daemonize code
+ *
+ * Revision 2.15  2003/03/01 15:55:02  gul
+ * Current outgoing address is now attibute of session, but not node
+ *
+ * Revision 2.14  2003/02/28 20:39:08  gul
+ * Code cleanup:
+ * change "()" to "(void)" in function declarations;
+ * change C++-style comments to C-style
+ *
+ * Revision 2.13  2003/02/22 20:19:54  gul
+ * Update copyrightes, 2002->2003
+ *
+ * Revision 2.12  2003/02/22 14:30:18  gul
+ * Make nNod var static
+ *
+ * Revision 2.11  2003/02/21 16:47:57  gul
+ * Remove <socks.h> file and USE_SOCKS macro
+ *
+ * Revision 2.10  2002/12/10 22:05:29  gul
+ * binkd/win32 bugfix
+ *
+ * Revision 2.9  2002/11/13 07:49:27  gul
+ * Fix typo
+ *
+ * Revision 2.8  2002/11/12 16:55:58  gul
+ * Run as service under win9x
+ *
+ * Revision 2.7  2002/07/22 19:48:30  gul
+ * Update usage
+ *
+ * Revision 2.6  2001/11/07 13:46:20  gul
+ * bugfix (added saved_argc)
+ *
+ * Revision 2.5  2001/10/29 19:31:44  gul
+ * Added -h option (print usage)
+ *
+ * Revision 2.4  2001/10/28 14:52:58  gul
+ * Cyrillic filenames fix
+ *
+ * Revision 2.3  2001/08/24 13:23:28  da
+ * binkd/binkd.c
+ * binkd/readcfg.c
+ * binkd/readcfg.h
+ * binkd/server.c
+ * binkd/nt/service.c
+ *
+ * Revision 2.2  2001/05/23 15:44:09  gul
+ * help-message fixed (-C option)
+ *
+ * Revision 2.1  2001/01/16 03:53:23  gul
  * Added -D switch (run as daemon)
  *
  * Revision 2.0  2001/01/10 12:12:37  gul
@@ -65,13 +180,13 @@
 #include <errno.h>
 #include <signal.h>
 #include <setjmp.h>
-#if defined(__IBMC__) || defined(__MSC__)
 #include <time.h>
-#endif
+#include <ctype.h>
 
 #include "Config.h"
 #include "sys.h"
 #include "iphdr.h"
+#include "common.h"
 #include "readcfg.h"
 #include "server.h"
 #include "client.h"
@@ -81,21 +196,26 @@
 #include "assert.h"
 #include "binlog.h"
 #include "setpttl.h"
-#ifndef DOS
-#include "daemonize.h"
+#include "sem.h"
+#ifdef UNIX
+#include "unix/daemonize.h"
 #endif
 
 #ifdef WIN32
+#include <windows.h>
 #include "nt/service.h"
+#ifdef BINKDW9X
+#include "nt/win9x.h"
 #endif
-
-#ifdef USE_SOCKS
-#include "socks.h"
 #endif
 
 #ifdef HAVE_THREADS
 #include "sem.h"
-MUTEXSEM hostsem = 0; 
+MUTEXSEM hostsem = 0;
+MUTEXSEM resolvsem = 0;
+MUTEXSEM varsem = 0;
+EVENTSEM eothread = 0;
+EVENTSEM exitcmgr = 0;
 #ifdef OS2
 MUTEXSEM fhsem = 0;
 #endif
@@ -124,65 +244,87 @@ static void hup (int signo)
 }
 #endif
 
-void usage ()
+void usage (void)
 {
-#if defined(WIN32)	
+#if defined(WIN32) && !defined(BINKDW9X)
 	char *s=NULL;
-	switch(checkservice())
+	if(checkservice() > 0)
 	{
-	case 1: s="i\0  -i       install WindowsNT service\n"; break;
-	case 2: s="u\0  -u       UNinstall WindowsNT service\n"; break;
+	  s="iu\0  -i       install WindowsNT service\n"
+                "  -u       uninstall WindowsNT service\n"
+                "  -S srvn  name of WindowsNT service (default: "
+                DEFAULT_SRVNAME ")"
+                "\n";
 	}
 #endif
 
-  printf ("usage: binkd [-Cc"
+#if defined(BINKDW9X)
+	AllocTempConsole();
+#endif
+
+  printf ("usage: binkd [-Ccpqrsvmh"
 #if defined(HAVE_DAEMON) || defined(HAVE_SETSID) || defined(HAVE_TIOCNOTTY)
           "D"
 #endif
 #if defined(UNIX) || defined(OS2) || defined(AMIGA)
 	  "i"
+#elif defined(BINKDW9X)
+	  "iu] [-S name] [-t cmd"
 #elif defined(WIN32)
-	  "%s"
+	  "T%s] [-S srvn"
 #endif
-	  "pqsv] [-Pnode] config"
+	  "] [-P node] config"
 #ifdef OS2
 	  " [socket]"
 #endif
 	  "\n"
-#if defined(HAVE_DAEMON) || defined(HAVE_SETSID) || defined(HAVE_TIOCNOTTY)
+#ifdef BINKD_DAEMONIZE
 	  "  -D       run as daemon\n"
 #endif
+#if defined(HAVE_FORK) || defined(BINKDW9X)
+	  "  -C       reload on config change\n"
+#else
 	  "  -C       exit(3) on config change\n"
+#endif
 	  "  -c       run client only\n"
 #if defined(UNIX) || defined(OS2) || defined(AMIGA)
 	  "  -i       run from inetd\n"
+#elif defined(BINKDW9X)
+	  "  -i       install Win9x service\n"
+	  "  -u       uninstall Win9x service\n"
+	  "  -t cmd   (start|stop|restart|status) service(s)\n"
+	  "  -S name  set Win9x service name, all - use all services\n"
 #elif defined(WIN32)
+	  "  -T       minimize to System Tray\n"
 	  "%s"
 #endif
 	  "  -P node  poll a node\n"
 	  "  -p       run client only, poll, quit\n"
 	  "  -q       be quiet\n"
+	  "  -r       disable crypt traffic\n"
 	  "  -s       run server only\n"
 	  "  -v       be verbose / dump version and quit\n"
 	  "  -m       disable CRAM-MD5 authorization\n"
+	  "  -h       print this help\n"
 	  "\n"
-	  "Copyright (c) 1996-2000 Dima Maloff and others.\n"
-	  "\n"
+	  "Copyright (c) 1996-2003 Dima Maloff and others.\n"
+#if defined(WIN32) && !defined(BINKDW9X)
+	  ,s?s:"", s?s+3:""
+#endif
+	  );
+
+  puts ("\n"
     "This program is free software; you can redistribute it and/or modify\n"
     "it under the terms of the GNU General Public License as published by\n"
 	  "the Free Software Foundation. See COPYING.\n"
 	  "\n"
-	  "Report bugs to 2:463/68 or binkd-bugs@happy.kiev.ua.\n"
-#if defined(WIN32)	
-	  ,s?s:"", s?s+2:""
-#endif
-	  );
+	  "Report bugs to 2:463/68 or binkd-bugs@happy.kiev.ua.");
   exit (1);
 }
 
 /* Command line flags */
 int inetd_flag = 0;		       /* Run from inetd (-i) */
-#if defined(HAVE_DAEMON) || defined(HAVE_SETSID) || defined(HAVE_TIOCNOTTY)
+#ifdef BINKD_DAEMONIZE
 int daemon_flag = 0;		       /* Run as daemon (-D) */
 #endif
 int server_flag = 0;		       /* Run servermgr (-s) */
@@ -193,29 +335,44 @@ int quiet_flag = 0;		       /* Be quiet (-q) */
 int verbose_flag = 0;		       /* Be verbose / print version (-v) */
 int checkcfg_flag = 0;		       /* exit(3) on config change (-C) */
 int no_MD5 = 0;			       /* disable MD5 flag (-m) */
+int no_crypt = 0;		       /* disable CRYPT (-r) */
 
-extern int nNod;
+#ifdef BINKDW9X
+extern const char *Win9xStartService;  /* 'Run as win9x service' option */
+#endif
 
+#ifdef BINKDW9X
+int binkd_main (int argc, char *argv[], char *envp[])
+#else
 int main (int argc, char *argv[], char *envp[])
+#endif
 {
   char tmp[128];
   int i;
 /* Config file name */
   char *config = NULL;
   char **saved_argv;
+  int saved_argc;
 #if defined(HAVE_DAEMON) || defined(HAVE_SETSID) || defined(HAVE_TIOCNOTTY)
   int  nochdir;
 #endif
 
-#if defined(__IBMC__) || defined(__MSC__)
   tzset();
-#endif
-
-#ifdef WIN32
+#if defined(WIN32) && !defined(BINKDW9X)
   service(argc, argv, envp);
 #endif
   /* save argv as setproctitle() under some systems will change it */
   saved_argv = mkargv (argc, argv);
+  saved_argc = argc;
+
+  InitSem (&hostsem);
+  InitSem (&resolvsem);
+  InitSem (&varsem);
+  InitEventSem (&eothread);
+  InitEventSem (&exitcmgr);
+#ifdef OS2
+  InitSem (&fhsem);
+#endif
 
   for (i = 1; i < argc; ++i)
   {
@@ -230,9 +387,19 @@ int main (int argc, char *argv[], char *envp[])
 	    case '-':
 	      /* GNU-style options */
 	      if (!strcmp (s + 1, "help"))
+	      {
+#if defined(WIN32) && !defined (BINKDW9X)
+              if (!isService)
+#endif
 		usage ();
+              }
+#if defined (BINKDW9X)
 	      else
-		Log (0, "%s: --%s: unknown command line switch", argv[0], s + 1);
+	      if (!strcmp(argv[i], Win9xStartService))
+	        goto BREAK_WHILE;
+#endif
+	      else
+		Log (0, "%s: --%s: unknown command line switch", extract_filename(argv[0]), s + 1);
 	    case 'C':
 	      checkcfg_flag = 1;
 	      break;
@@ -243,13 +410,25 @@ int main (int argc, char *argv[], char *envp[])
 	    case 'i':
 	      inetd_flag = 1;
 	      break;
+#endif
+#if defined (WIN32)
+#if !defined (BINKDW9X)
+	    case 'T': /* Skip option in win32 version */
+	      break;
+#endif
+	    case 'S': /* Skip option and parameter in all win* versions */
+#if defined (BINKDW9X)
+            case 't': /* Skip option and parameter in win9x version */
+#endif
+	      if (!s[1]){ ++i; }
+	      goto BREAK_WHILE;
 #endif
 	    case 'P':
 	      if (argv[i][2] == 0)
 	      {
 		++i;
 		if (argv[i] == 0)
-		  Log (0, "%s: -P: missing requred argument", argv[0]);
+		  Log (0, "%s: -P: missing requred argument", extract_filename(argv[0]));
 	      }
 	      goto BREAK_WHILE;
 	    case 'p':
@@ -263,11 +442,14 @@ int main (int argc, char *argv[], char *envp[])
 	      break;
 	    case 'm':
 	      no_MD5 = 1;
+	      /* fallthrough */
+	    case 'r':
+	      no_crypt = 1;
 	      break;
 	    case 'v':
 	      ++verbose_flag;
 	      break;
-#if defined(HAVE_DAEMON) || defined(HAVE_SETSID) || defined(HAVE_TIOCNOTTY)
+#ifdef BINKD_DAEMONIZE
 	    case 'D':
 	      daemon_flag = 1;
 	      /* remove this switch from saved_argv */
@@ -275,13 +457,24 @@ int main (int argc, char *argv[], char *envp[])
 	        free(saved_argv[i]);
 	        for (j=i; j<argc; j++)
 	          saved_argv[j]=saved_argv[j+1];
+	        saved_argc--;
 	      }
 	      break;
 #endif
+	    case 'h':
+#if defined(WIN32) && !defined(BINKDW9X)
+              if (isService)
+                 break;
+#endif
+              usage();
 	    default:
-	      Log (0, "%s: -%c: unknown command line switch", argv[0], *s);
-	    case 0:
-	      usage ();
+	      Log (0, "%s: -%c: unknown command line switch", extract_filename(argv[0]), *s);
+	    case '\0':
+#if defined(WIN32) && !defined (BINKDW9X)
+              if (isService)
+                 break;
+#endif
+              usage();
 	  }
 	++s;
       }
@@ -326,6 +519,10 @@ int main (int argc, char *argv[], char *envp[])
     readcfg (config);
   else if (verbose_flag)
   {
+#if defined(WIN32) && defined(BINKDW9X)
+    AllocTempConsole();
+#endif
+
     printf ("Binkd " MYVER " (" __DATE__ " " __TIME__ "%s)\n", get_os_string ());
     exit (0);
   }
@@ -333,7 +530,7 @@ int main (int argc, char *argv[], char *envp[])
     usage ();
   else
   {
-    Log (0, "%s: invalid command line: config name must be specified", argv[0]);
+    Log (0, "%s: invalid command line: config name must be specified", extract_filename(argv[0]));
     exit (1);
   }
 
@@ -360,7 +557,7 @@ int main (int argc, char *argv[], char *envp[])
     }
 
   print_args (tmp, sizeof (tmp), argc - 1, argv + 1);
-  Log (4, "BEGIN, " MYNAME "/" MYVER "%s", tmp);
+  Log (4, "BEGIN, " MYNAME "/" MYVER "%s%s", get_os_string(), tmp);
 
   if (sock_init ())
     Log (0, "sock_init: %s", TCPERR ());
@@ -369,6 +566,9 @@ int main (int argc, char *argv[], char *envp[])
   BinLogInit ();
   rnd ();
   initsetproctitle (argc, argv, envp);
+#ifdef WIN32
+  SetFileApisToOEM();
+#endif
 
   /* Set up break handler, set up exit list if needed */
   if (!set_break_handlers ())
@@ -376,11 +576,6 @@ int main (int argc, char *argv[], char *envp[])
 
 #ifdef HAVE_FORK
   signal (SIGCHLD, chld);
-#else
-  InitSem (&hostsem);
-#ifdef OS2
-  InitSem (&fhsem);
-#endif
 #endif
 
   for (i = 1; i < argc; ++i)
@@ -395,13 +590,13 @@ int main (int argc, char *argv[], char *envp[])
 #if defined(UNIX) || defined(OS2) || defined(AMIGA)
   if (inetd_flag)
   {
-    protocol (inetd_socket, 0);
+    protocol (inetd_socket, 0, NULL);
     soclose (inetd_socket);
     exit (0);
   }
 #endif
 
-#if defined(HAVE_DAEMON) || defined(HAVE_SETSID) || defined(HAVE_TIOCNOTTY)
+#ifdef BINKD_DAEMONIZE
   if (daemon_flag)
   {
 	 if (!server_flag)
@@ -413,7 +608,7 @@ int main (int argc, char *argv[], char *envp[])
 		 else
 		 {
 			nochdir = 1;
-		 	// Log (6, "Run with relative path, will not chdir to /");
+		 	/* Log (6, "Run with relative path, will not chdir to /"); */
 		 }
 
 		 if (binkd_daemonize(nochdir) < 0)
@@ -443,13 +638,9 @@ int main (int argc, char *argv[], char *envp[])
 #if defined(UNIX) || defined(AMIGA) || (defined(OS2) && defined(HAVE_FORK))
   if (setjmp (jb))
   {
-    extern SOCKET sockfd;
-
-    Log (5, "Closing socket # %i", sockfd);
-    soclose (sockfd);
 binkdrestart:
     exitfunc();
-    print_args (tmp, sizeof (tmp), argc - 1, saved_argv + 1);
+    print_args (tmp, sizeof (tmp), saved_argc - 1, saved_argv + 1);
     Log (2, "exec %s%s", saved_argv[0], tmp);
     if (execv (saved_argv[0], saved_argv) == -1)
       Log (1, "execv: %s", strerror (errno));
