@@ -12,9 +12,77 @@
  */
 
 /*
- * $Id: ftnq.c,v 2.0 2001/01/10 12:12:38 gul Exp $
+ * $Id: ftnq.c,v 2.19.2.3 2003/10/19 12:51:42 stream Exp $
  *
  * $Log: ftnq.c,v $
+ * Revision 2.19.2.3  2003/10/19 12:51:42  stream
+ * Missing return value for q_scan_boxes()
+ *
+ * Revision 2.19.2.2  2003/09/15 19:57:40  gul
+ * Fixed double-locking pNod array
+ *
+ * Revision 2.19.2.1  2003/09/14 12:20:05  gul
+ * Clean use pointers to pNod array
+ *
+ * Revision 2.19  2003/05/28 14:32:57  gul
+ * new function q_add_last_file() - add file to the end of queue
+ *
+ * Revision 2.18  2003/04/24 20:07:38  gul
+ * Send freq first
+ *
+ * Revision 2.17  2003/04/22 22:30:18  gul
+ * rtl workaround, incorrect diagnostics if poll already exists
+ *
+ * Revision 2.16  2003/04/02 13:12:57  gul
+ * Try to use workaround for buggy windows time functions (timezone)
+ *
+ * Revision 2.15  2003/03/31 20:28:24  gul
+ * safe_localtime() and safe_gmtime() functions
+ *
+ * Revision 2.14  2003/03/25 13:45:16  gul
+ * Case-insencitive t-mail "hold" long filebox search
+ *
+ * Revision 2.13  2003/03/25 09:18:54  gul
+ * Case-insencitive t-mail "hold" long filebox search
+ *
+ * Revision 2.12  2003/03/10 12:16:53  gul
+ * Use HAVE_DOS_H macro
+ *
+ * Revision 2.11  2003/03/10 10:57:45  gul
+ * Extern declarations moved to header files
+ *
+ * Revision 2.10  2003/03/05 13:21:50  gul
+ * Fix warnings
+ *
+ * Revision 2.9  2003/03/01 20:16:27  gul
+ * OS/2 IBM C support
+ *
+ * Revision 2.8  2003/03/01 15:26:50  gul
+ * *** empty log message ***
+ *
+ * Revision 2.7  2003/03/01 15:17:48  gul
+ * *** empty log message ***
+ *
+ * Revision 2.6  2003/02/28 20:39:08  gul
+ * Code cleanup:
+ * change "()" to "(void)" in function declarations;
+ * change C++-style comments to C-style
+ *
+ * Revision 2.5  2003/02/28 19:40:47  gul
+ * Outbound scan bugfix if defnode used
+ *
+ * Revision 2.4  2003/02/22 21:32:46  gul
+ * Amiga Style Outbound support
+ *
+ * Revision 2.3  2003/02/22 15:53:46  gul
+ * Bugfix with locking array of nodes in multithread version
+ *
+ * Revision 2.2  2003/02/22 12:56:00  gul
+ * Do not give unsecure mail to secuse link when send-if-pwd
+ *
+ * Revision 2.1  2002/03/15 15:49:30  gul
+ * Do not check/create *.try if "try" is not defined in config
+ *
  * Revision 2.0  2001/01/10 12:12:38  gul
  * Binkd is under CVS again
  *
@@ -69,6 +137,7 @@ static const char out_flvrs[] = "icdohICDOH";
 
 static FTNQ *q_add_dir (FTNQ *q, char *dir, FTN_ADDR *fa1);
 FTNQ *q_add_file (FTNQ *q, char *filename, FTN_ADDR *fa1, char flvr, char action, char type);
+static FTNQ *q_scan_boxes_nolock (FTNQ *q, FTN_ADDR *fa, int n);
 
 /*
  * q_free(): frees memory allocated by q_scan()
@@ -116,10 +185,7 @@ void q_free (FTNQ *q)
  */
 static int qn_scan (FTN_NODE *fn, void *arg)
 {
-  char buf[FTN_ADDR_SZ + 1];
-
-  ftnaddress_to_str (buf, &fn->fa);
-  *(FTNQ **) arg = q_scan_boxes (*(FTNQ **) arg, &fn->fa, 1);
+  *(FTNQ **) arg = q_scan_boxes_nolock (*(FTNQ **) arg, &fn->fa, 1);
   return 0;
 }
 
@@ -127,8 +193,6 @@ FTNQ *q_scan (FTNQ *q)
 {
   char *s;
   char buf[MAXPATHLEN + 1], outb_path[MAXPATHLEN + 1];
-
-  extern FTN_DOMAIN *pDomains;
   FTN_DOMAIN *curr_domain;
 
   for (curr_domain = pDomains; curr_domain; curr_domain = curr_domain->next)
@@ -166,9 +230,12 @@ FTNQ *q_scan (FTNQ *q)
 	  FTN_ADDR fa;
 
 	  FA_ZERO (&fa);
-	  fa.z = ((de->d_name[len] == '.') ?
-		  (int)strtol (de->d_name + len + 1, (char **) NULL, 16) :
-		  curr_domain->z[0]);
+#ifdef AMIGADOS_4D_OUTBOUND
+	  if (!aso)
+#endif
+	    fa.z = ((de->d_name[len] == '.') ?
+		    strtol (de->d_name + len + 1, (char **) NULL, 16) :
+		    curr_domain->z[0]);
 	  if (de->d_name[len] == 0 || fa.z != curr_domain->z[0])
 	  {
 	    strcpy (fa.domain, curr_domain->name);
@@ -188,14 +255,26 @@ FTNQ *q_scan (FTNQ *q)
 /*
  * Adds to the q all files for n akas stored in fa
  */
-FTNQ *q_scan_addrs (FTNQ *q, FTN_ADDR *fa, int n)
+FTNQ *q_scan_addrs (FTNQ *q, FTN_ADDR *fa, int n, int to)
 {
   char buf[MAXPATHLEN + 1];
-  int i;
+  int  i;
   char *s;
 
   for (i = 0; i < n; ++i)
   {
+    if (!to && send_if_pwd)
+    {
+      /* do not give unsecure mail even to secure link when send-if-pwd */
+      FTN_NODE *fn;
+      locknodesem();
+      if ((fn = get_node_info(fa+i)) == NULL ||
+           fn->pwd == NULL || strcmp(fn->pwd, "-") == 0)
+      { releasenodesem();
+	continue;
+      }
+      releasenodesem();
+    }
     ftnaddress_to_filename (buf, fa + i);
     if (*buf)
     {
@@ -227,7 +306,7 @@ static int cmp_filebox_files (FTNQ *a, FTNQ *b)
   int w_b = weight_by_name (b->path);
 
   if (w_a - w_b == 0)
-    return (int)(b->time - a->time);
+    return b->time - a->time;
   else
     return w_a - w_b;
 }
@@ -235,10 +314,10 @@ static int cmp_filebox_files (FTNQ *a, FTNQ *b)
 #ifdef MAILBOX
 static char to32(int N)
 {
-    if ((N >= 0) && (N <=9)) return (char)('0'+N);
-    if ((N > 9) && (N < 32)) return (char)('A'+ N - 10);
+    if ((N >= 0) && (N <=9)) return '0'+N;
+    if ((N > 9) && (N < 32)) return 'A'+ N - 10;
     return 'Z'; /* fake return */
-};
+}
 
 static struct {
     char * ext;
@@ -279,7 +358,7 @@ static FTNQ *q_scan_box (FTNQ *q, FTN_ADDR *fa, char *boxpath, char flvr, int de
       strnzcat (buf, de->d_name, sizeof (buf));
       if (stat (buf, &sb) == 0 && (sb.st_mode & S_IFDIR) == 0 &&
 	  de->d_name[0] != '.'
-#if defined(OS2) && !defined(__IBMC__)
+#if defined(OS2) && !defined(IBMC) && !defined(__WATCOMC__)
           && (de->d_attr & 0x02) == 0   /* not hidden */
 #endif
 	 )
@@ -315,7 +394,7 @@ static FTNQ *q_scan_box (FTNQ *q, FTN_ADDR *fa, char *boxpath, char flvr, int de
 /*
  * Scans fileboxes for n akas stored in fa
  */
-FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
+static FTNQ *q_scan_boxes_nolock (FTNQ *q, FTN_ADDR *fa, int n)
 {
   FTN_NODE *node;
   int i;
@@ -373,7 +452,13 @@ FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
                 node->fa.node,
                 node->fa.p);
         q = q_scan_box (q, fa+i, buf, 'f', deleteablebox);
-        strnzcat ( buf, ".h", sizeof (buf));
+        strnzcat ( buf, ".H", sizeof (buf));
+#ifdef UNIX
+	{ struct stat st;
+	  if (stat(buf, &st) != 0 || (st.st_mode & S_IFDIR) == 0)
+	    buf[strlen(buf) - 1] = 'h';
+	}
+#endif
         q = q_scan_box (q, fa+i, buf, 'h', deleteablebox);
 
         strnzcpy ( buf, tfilebox, sizeof (buf));
@@ -389,7 +474,13 @@ FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
                 to32(node->fa.node/1024),to32((node->fa.node/32)%32),to32(node->fa.node%32),
                 to32(node->fa.p/32),     to32(node->fa.p%32));
         q = q_scan_box (q, fa+i, buf, 'f', deleteablebox);
-        strnzcat (buf, "h", sizeof (buf));
+        strnzcat (buf, "H", sizeof (buf));
+#ifdef UNIX
+	{ struct stat st;
+	  if (stat(buf, &st) != 0 || (st.st_mode & S_IFDIR) == 0)
+	    buf[strlen(buf) - 1] = 'h';
+	}
+#endif
         q = q_scan_box (q, fa+i, buf, 'h', deleteablebox);
       }
     }
@@ -399,13 +490,21 @@ FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
   return q;
 }
 
+FTNQ *q_scan_boxes (FTNQ *q, FTN_ADDR *fa, int n)
+{
+  locknodesem();
+  q = q_scan_boxes_nolock (q, fa, n);
+  releasenodesem();
+  return q;
+}
 
 void process_hld (FTN_ADDR *fa, char *path)
 {
-  FTN_NODE *node = get_node_info (fa);
+  FTN_NODE *node;
   long hold_until_tmp;
 
-  if (node)
+  locknodesem();
+  if ((node = get_node_info(fa)) != NULL)
   {
     FILE *f;
 
@@ -418,12 +517,13 @@ void process_hld (FTN_ADDR *fa, char *path)
     if (f)
       fclose (f);
 
-    if (node->hold_until <= time (0))
+    if (node->hold_until <= safe_time())
     {
       node->hold_until = 0;
       delete (path);
     }
   }
+  releasenodesem();
 }
 
 void process_bsy (FTN_ADDR *fa, char *path)
@@ -441,10 +541,15 @@ void process_bsy (FTN_ADDR *fa, char *path)
     Log (2, "found old %s file for %s", s, buf);
     delete (path);
   }
-  else if ((node = get_node_info (fa)) != 0 && node->busy != 'b' &&
-	   (!STRICMP (s, ".bsy") || !STRICMP (s, ".csy")))
+  else
   {
-    node->busy = tolower (s[1]);
+    locknodesem();
+    if ((node = get_node_info (fa)) != 0 && node->busy != 'b' &&
+	   (!STRICMP (s, ".bsy") || !STRICMP (s, ".csy")))
+    {
+      node->busy = tolower (s[1]);
+    }
+    releasenodesem();
   }
 }
 
@@ -471,76 +576,136 @@ static FTNQ *q_add_dir (FTNQ *q, char *dir, FTN_ADDR *fa1)
 
     while ((de = readdir (dp)) != 0)
     {
-      s = de->d_name;
-
-      for (j = 0; j < 8; ++j)
-	if (!isxdigit (s[j]))
-	  break;
-
-      if (j == 8 && strchr (s, 0) - s == 12 &&
-	  strchr (s, '.') - s == 8 && strrchr (s, '.') - s == 8)
+#ifdef AMIGADOS_4D_OUTBOUND
+      if (aso)
       {
+        char ext[4];
+        int matched = 0;
+        size_t nlen = strlen(s = de->d_name);
+
+	for (; *s && isgraph(*s) != 0; s++);
+	if ((size_t)(s - de->d_name) != nlen)
+	  continue;
+
+        memcpy (&fa2, fa1, sizeof(FTN_ADDR));
+
+        if (sscanf(s = de->d_name, "%u.%u.%u.%u.%3s%n",
+	         &fa2.z, &fa2.net, &fa2.node, &fa2.p, ext, &matched) != 5 ||
+	    (size_t)matched != nlen || strlen(ext) != 3)
+	  continue;
+
+        if ((fa1->z != -1 && fa1->z != fa2.z) ||
+	    (fa1->net != -1 && fa1->net != fa2.net) ||
+	    (fa1->node != -1 && fa1->node != fa2.node) ||
+	    (fa1->p != -1 && fa1->p != fa2.p))
+	  continue;
+
+        strnzcpy(buf, dir, sizeof(buf));
+        strnzcpy(buf + strlen(buf), PATH_SEPARATOR, sizeof(buf) - strlen(buf));
+        strnzcpy(buf + strlen(buf), s, sizeof(buf) - strlen(buf));
+
+        if (!STRICMP(ext, "bsy") || !STRICMP(ext, "csy"))
+	  process_bsy(&fa2, buf);
+
+	locknodesem();
+        if (!get_node_info(&fa2) && !is5D(fa1))
+	{ releasenodesem();
+          continue;
+	}
+	releasenodesem();
+
+        if (strchr(out_flvrs, ext[0]) &&
+		  tolower(ext[1]) == 'u' && tolower(ext[2]) == 't')
+	  /* Adding *.?ut */
+	  q = q_add_file(q, buf, &fa2, ext[0], 'd', 'm');
+        else if (!STRICMP(ext, "req"))
+	  /* Adding *.req */
+	  q = q_add_file(q, buf, &fa2, 'h', 's', 'r');
+        else if (!STRICMP(ext, "hld"))
+	  process_hld(&fa2, buf);
+        else if (strchr(flo_flvrs, ext[0]) &&
+		        tolower(ext[1]) == 'l' && tolower(ext[2]) == 'o')
+	  /* Adding *.?lo */
+	  q = q_add_file(q, buf, &fa2, ext[0], 'd', 'l');
+	else if (!STRICMP (s + 9, "stc"))
+	{
+	  /* Adding *.stc */
+	  q = q_add_file (q, buf, &fa2, 'h', 0, 's');
+	}
+      }
+      else
+#endif /* AMIGADOS_4D_OUTBOUND */
+      {
+        s = de->d_name;
+
+        for (j = 0; j < 8; ++j)
+	  if (!isxdigit (s[j]))
+	    break;
+
+        if (j != 8 || strlen(s) != 12 || s[8] != '.' || strchr(s+9, '.'))
+	  continue;
+
 	/* fa2 will store dest.address for the current (de->d_name) file */
 	memcpy (&fa2, fa1, sizeof (FTN_ADDR));
 
 	if (fa1->node != -1 && fa1->p != 0)
-	  sscanf (s, "%8x", &fa2.p);   /* We now in /xxxxyyyy.pnt */
+	  sscanf (s, "%8x", (unsigned *)&fa2.p);   /* We now in /xxxxyyyy.pnt */
 	else
-	  sscanf (s, "%4x%4x", &fa2.net, &fa2.node);
+	  sscanf (s, "%4x%4x", (unsigned *)&fa2.net, (unsigned *)&fa2.node);
 
 	/* add the file if wildcard (f1) match the address (fa2) */
-	if (fa1->node == -1 || fa1->p == -1 || !ftnaddress_cmp (fa1, &fa2))
+	if (fa1->node != -1 && fa1->p != -1 && ftnaddress_cmp (fa1, &fa2))
+	  continue;
+
+	strnzcpy (buf, dir, sizeof (buf));
+	strnzcpy (buf + strlen (buf), PATH_SEPARATOR,
+		  sizeof (buf) - strlen (buf));
+	strnzcpy (buf + strlen (buf), s, sizeof (buf) - strlen (buf));
+
+	if (!STRICMP (s + 9, "pnt") && fa2.p == -1)
 	{
-	  strnzcpy (buf, dir, sizeof (buf));
-	  strnzcpy (buf + strlen (buf), PATH_SEPARATOR,
-		    sizeof (buf) - strlen (buf));
-	  strnzcpy (buf + strlen (buf), s, sizeof (buf) - strlen (buf));
+	  struct stat sb;
 
-	  if (!STRICMP (s + 9, "pnt") && fa2.p == -1)
-	  {
-	    struct stat sb;
+	  if (stat (buf, &sb) == 0 && sb.st_mode & S_IFDIR)
+	    q = q_add_dir (q, buf, &fa2);
+	  continue;
+	}
+	if (fa2.p == -1)
+	  fa2.p = 0;
 
-	    if (stat (buf, &sb) == 0 && sb.st_mode & S_IFDIR)
-	      q = q_add_dir (q, buf, &fa2);
-	  }
-	  else
-	  {
-	    if (fa2.p == -1)
-	      fa2.p = 0;
+	if (!STRICMP (s + 9, "bsy") || !STRICMP (s + 9, "csy"))
+	  process_bsy (&fa2, buf);
 
-	    if (!STRICMP (s + 9, "bsy") || !STRICMP (s + 9, "csy"))
-	      process_bsy (&fa2, buf);
-
-	    if (get_node_info (&fa2) || is5D (fa1))
-	    {
-	      if (strchr (out_flvrs, s[9]) &&
-		  tolower (s[10]) == 'u' && tolower (s[11]) == 't')
-	      {
-		/* Adding *.?ut */
-		q = q_add_file (q, buf, &fa2, s[9], 'd', 'm');
-	      }
-	      else if (!STRICMP (s + 9, "req"))
-	      {
-		/* Adding *.req */
-		q = q_add_file (q, buf, &fa2, 'h', 's', 'r');
-	      }
-	      else if (!STRICMP (s + 9, "hld"))
-	      {
-		process_hld (&fa2, buf);
-	      }
-	      else if (strchr (flo_flvrs, s[9]) &&
-		       tolower (s[10]) == 'l' && tolower (s[11]) == 'o')
-	      {
-		/* Adding *.?lo */
-		q = q_add_file (q, buf, &fa2, s[9], 'd', 'l');
-	      }
-	      else if (!STRICMP (s + 9, "stc"))
-	      {
-		/* Adding *.stc */
-		q = q_add_file (q, buf, &fa2, 'h', 0, 's');
-	      }
-	    }
-	  }
+	locknodesem();
+	if (!havedefnode && !get_node_info (&fa2) && !is5D (fa1))
+	{
+	  releasenodesem();
+	  continue;
+	}
+	releasenodesem();
+	if (strchr (out_flvrs, s[9]) &&
+	    tolower (s[10]) == 'u' && tolower (s[11]) == 't')
+	{
+	  /* Adding *.?ut */
+	  q = q_add_file (q, buf, &fa2, s[9], 'd', 'm');
+	}
+	else if (!STRICMP (s + 9, "req"))
+	{
+	  /* Adding *.req */
+	  q = q_add_file (q, buf, &fa2, 'h', 's', 'r');
+	}
+	else if (!STRICMP (s + 9, "hld"))
+	  process_hld (&fa2, buf);
+	else if (strchr (flo_flvrs, s[9]) &&
+	         tolower (s[10]) == 'l' && tolower (s[11]) == 'o')
+	{
+	  /* Adding *.?lo */
+	  q = q_add_file (q, buf, &fa2, s[9], 'd', 'l');
+	}
+	else if (!STRICMP (s + 9, "stc"))
+	{
+	  /* Adding *.stc */
+	  q = q_add_file (q, buf, &fa2, 'h', 0, 's');
 	}
       }
     }
@@ -617,19 +782,37 @@ FTNQ *q_add_file (FTNQ *q, char *filename, FTN_ADDR *fa1, char flvr, char action
   }
   else
   {
-    FTN_NODE *node = get_node_info (fa1);
+    FTN_NODE *node;
 
-    if (node)
+    locknodesem();
+    if ((node = get_node_info (fa1)) != NULL)
     {
       if (type == 'm')
 	node->mail_flvr = MAXFLVR (flvr, node->mail_flvr);
       else
 	node->files_flvr = MAXFLVR (flvr, node->files_flvr);
     }
+    releasenodesem();
   }
   return q;
 }
 
+/*
+ * Add a file to the end of queue.
+ */
+FTNQ *q_add_last_file (FTNQ *q, char *filename, FTN_ADDR *fa1, char flvr, char action, char type)
+{
+  FTNQ *new_file, *pq;
+
+  new_file = q_add_file (NULL, filename, fa1, flvr, action, type);
+  if (new_file == NULL) return q;
+  if (q == NULL) return new_file;
+  for (pq = q; pq->next; pq = pq->next);
+  new_file->prev = pq;
+  pq->next = new_file;
+  return q;
+}
+
 /*
  * q_list: just lists q, not more
  */
@@ -649,15 +832,9 @@ static int qn_list (FTN_NODE *fn, void *arg)
   {
     if (fn->hold_until > 0)
     {
-      struct tm *tm;
-#ifdef __WATCOMC__
-      struct tm stm;
-      tm = &stm;
-      _localtime (&fn->hold_until, tm);
-#else
-      tm = localtime (&fn->hold_until);
-#endif
-      strftime (tmp, sizeof (tmp), " (hold until %Y/%m/%d %H:%M:%S)", tm);
+      struct tm tm;
+      safe_localtime (&fn->hold_until, &tm);
+      strftime (tmp, sizeof (tmp), " (hold until %Y/%m/%d %H:%M:%S)", &tm);
     }
     else
       *tmp = 0;
@@ -722,9 +899,19 @@ FTNQ *select_next_file (FTNQ *q, FTN_ADDR *fa, int nAka)
 	return curr;
       }
     }
+    for (curr = q; curr; curr = curr->next)	/* Freq before netmail */
+    {
+      if (!curr->sent &&
+	  (FA_ISNULL (&curr->fa) || !ftnaddress_cmp (&curr->fa, fa + k)) &&
+	  curr->type == 'r')
+      {
+	curr->sent = 1;
+	return curr;
+      }
+    }
     for (j = 0; prio[j]; ++j)
     {
-      for (curr = q; curr; curr = curr->next)	/* Netmail first */
+      for (curr = q; curr; curr = curr->next)	/* Netmail before files */
       {
 	if (!curr->sent &&
 	    (FA_ISNULL (&curr->fa) || !ftnaddress_cmp (&curr->fa, fa + k)) &&
@@ -804,42 +991,41 @@ static int qn_not_empty (FTN_NODE *fn, void *arg)
 {
   qn_not_empty_arg *a = (qn_not_empty_arg *) arg;
 
-  if (!fn->busy && strcmp (fn->hosts, "-") && fn->hold_until < time (0))
+  if (!fn->busy && strcmp (fn->hosts, "-") && fn->hold_until < safe_time())
   {
     if (a->maxflvr != MAXFLVR (fn->mail_flvr, MAXFLVR (fn->files_flvr, a->maxflvr)))
     {
       a->maxflvr = MAXFLVR (fn->mail_flvr, fn->files_flvr);
-      a->fn = fn;
+      if (a->fn)
+        memcpy (a->fn, fn, sizeof (*fn));
     }
   }
   return 0;
 }
 
-FTN_NODE *q_not_empty ()
+int q_not_empty (FTN_NODE *fn)
 {
   qn_not_empty_arg arg;
 
   arg.maxflvr = 0;
-  arg.fn = 0;
+  arg.fn = fn;
 
   foreach_node (qn_not_empty, &arg);
 
   if (arg.maxflvr && tolower (arg.maxflvr) != 'h')
-    return arg.fn;
+    return -1;
   else
     return 0;
 }
 
-FTN_NODE *q_next_node ()
+int q_next_node (FTN_NODE *fn)
 {
-  FTN_NODE *fn = q_not_empty ();
-
-  if (fn == 0)
+  if (q_not_empty(fn) == 0)
     return 0;
   else
   {
     fn->mail_flvr = fn->files_flvr = 0;
-    return fn;
+    return -1;
   }
 }
 
@@ -851,15 +1037,17 @@ int create_poll (FTN_ADDR *fa, int flvr)
   char buf[MAXPATHLEN + 1];
   char ext[5];
   int rc = 0;
+  struct stat st;
 
   strcpy (ext, ".flo");
   if (flvr && strchr (flo_flvrs, flvr))
-    ext[1] = (char) tolower (flvr);
+    ext[1] = tolower (flvr);
   ftnaddress_to_filename (buf, fa);
   if (*buf)
   {
     mkpath (buf);
     strnzcat (buf, ext, sizeof (buf));
+    if (stat (buf, &st) == 0) return 1; /* already exists */
     if ((rc = create_empty_sem_file (buf)) == 0)
       if (errno != EEXIST)
         Log (1, "cannot create %s: %s", buf, strerror (errno));
@@ -877,31 +1065,27 @@ void hold_node (FTN_ADDR *fa, time_t hold_until)
   char buf[MAXPATHLEN + 1];
   char addr[FTN_ADDR_SZ + 1];
   char time[80];
-  struct tm *tm;
+  struct tm tm;
 
-#ifdef __WATCOMC__
-  struct tm stm;
-  tm = &stm;
-  _localtime (&hold_until, tm);
-#else
-  tm = localtime (&hold_until);
-#endif
-  strftime (time, sizeof (time), "%Y/%m/%d %H:%M:%S", tm);
+  safe_localtime (&hold_until, &tm);
+  strftime (time, sizeof (time), "%Y/%m/%d %H:%M:%S", &tm);
   ftnaddress_to_str (addr, fa);
   ftnaddress_to_filename (buf, fa);
   Log (2, "holding %s (%s)", addr, time);
   if (*buf)
   {
     FILE *f;
-    FTN_NODE *fn = get_node_info (fa);
+    FTN_NODE *fn;
 
     strnzcat (buf, ".hld", sizeof (buf));
     if ((f = fopen (buf, "w")) != NULL)
     {
       fprintf (f, "%li", (long int) hold_until);
-      if (fn)
-	fn->hold_until = hold_until;
       fclose (f);
+      locknodesem();
+      if ((fn = get_node_info (fa)) != NULL)
+	fn->hold_until = hold_until;
+      releasenodesem();
     }
     else
     {
@@ -965,11 +1149,12 @@ void bad_try (FTN_ADDR *fa, const char *error)
 {
   unsigned nok, nbad;
 
+  if (tries == 0) return;
   read_try (fa, &nok, &nbad);
   if (tries > 0 && ++nbad >= (unsigned) tries)
   {
     nok = nbad = 0;
-    hold_node (fa, time (0) + hold);
+    hold_node (fa, safe_time() + hold);
   }
   write_try (fa, &nok, &nbad, (char *) error);
 }
@@ -978,6 +1163,7 @@ void good_try (FTN_ADDR *fa, char *comment)
 {
   unsigned nok, nbad;
 
+  if (tries == 0) return;
   read_try (fa, &nok, &nbad);
   nbad = 0;
   ++nok;
