@@ -12,9 +12,51 @@
  */
 
 /*
- * $Id: inbound.c,v 2.1 2001/04/23 07:58:57 gul Exp $
+ * $Id: inbound.c,v 2.12.2.2 2003/08/28 07:38:09 gul Exp $
  *
  * $Log: inbound.c,v $
+ * Revision 2.12.2.2  2003/08/28 07:38:09  gul
+ * Cosmetics in log
+ *
+ * Revision 2.12.2.1  2003/08/27 06:46:37  gul
+ * Migrate from stable branch:
+ * remove partial if received part more then total size,
+ * flush buffer after receive data frame,
+ * drop session if extra bytes received.
+ *
+ * Revision 2.12  2003/03/10 10:39:23  gul
+ * New include file common.h
+ *
+ * Revision 2.11  2003/03/05 13:21:50  gul
+ * Fix warnings
+ *
+ * Revision 2.10  2003/03/01 18:29:52  gul
+ * Change size_t to off_t for file sizes and offsets
+ *
+ * Revision 2.9  2003/03/01 15:00:16  gul
+ * Join skipmask and overwrite into common maskchain
+ *
+ * Revision 2.8  2003/02/27 15:37:19  gul
+ * Bugfix in disk free space check
+ *
+ * Revision 2.7  2002/11/22 14:40:42  gul
+ * Check free space on inbox if defined
+ *
+ * Revision 2.6  2002/10/03 10:23:26  gul
+ * Check fprintf() & fclose() retcodes
+ *
+ * Revision 2.5  2002/07/22 19:38:23  gul
+ * overwrite minor fix
+ *
+ * Revision 2.4  2002/07/21 10:35:44  gul
+ * overwrite option
+ *
+ * Revision 2.3  2002/05/06 19:25:39  gul
+ * new keyword inboundCase in config
+ *
+ * Revision 2.2  2001/10/23 08:33:44  gul
+ * Change filename (not ext) in incoming *.req if already exists
+ *
  * Revision 2.1  2001/04/23 07:58:57  gul
  * getfree() on large drives fixed
  *
@@ -52,6 +94,7 @@
 
 #include "Config.h"
 #include "sys.h"
+#include "common.h"
 #include "readcfg.h"
 #include "inbound.h"
 #include "tools.h"
@@ -92,12 +135,24 @@ static int creat_tmp_name (char *s, char *file, off_t size,
       if ((f = fopen (s, "w")) == 0)
       {
 	Log (1, "%s: %s", s, strerror (errno));
+	delete (s);
 	return 0;
       }
       ftnaddress_to_str (node, from);
-      fprintf (f, "%s %li %li %s\n",
-	       file, (long int) size, (long int) time, node);
-      fclose (f);
+      if (fprintf (f, "%s %li %li %s\n",
+	           file, (long int) size, (long int) time, node) <= 0)
+      {
+	Log (1, "%s: %s", s, strerror (errno));
+	fclose (f);
+	delete (s);
+	return 0;
+      }
+      if (fclose (f))
+      {
+	Log (1, "%s: %s", s, strerror (errno));
+	delete (s);
+	return 0;
+      }
       break;
     }
     *t = 0;
@@ -183,7 +238,7 @@ int find_tmp_name (char *s, char *file, off_t size,
 	}
 	else if (kill_dup_partial_files && i < nfa)
 	{
-	  Log (4, "dup partial file (%s):", w[0]);
+	  Log (5, "dup partial file %s removed", w[0]);
 	  remove_hr (s);
 	}
       }
@@ -218,8 +273,8 @@ int find_tmp_name (char *s, char *file, off_t size,
   return found;
 }
 
-FILE *inb_fopen (char *netname, off_t size,
-		  time_t time, FTN_ADDR *from, int nfa, char *inbound)
+FILE *inb_fopen (char *netname, off_t size, time_t time, FTN_ADDR *from,
+                 int nfa, char *inbound, int secure_flag)
 {
   char buf[MAXPATHLEN + 1];
   struct stat sb;
@@ -228,8 +283,12 @@ FILE *inb_fopen (char *netname, off_t size,
   if (!find_tmp_name (buf, netname, size, time, from, nfa, inbound))
     return 0;
 
+fopen_again:
   if ((f = fopen (buf, "ab")) == 0)
+  {
     Log (1, "%s: %s", buf, strerror (errno));
+    return 0;
+  }
   fseek (f, 0, SEEK_END);	       /* Work-around MSVC bug */
 
 #if defined(OS2)
@@ -242,16 +301,27 @@ FILE *inb_fopen (char *netname, off_t size,
   if (fstat (fileno (f), &sb) == 0)
   {
     /* Free space req-d (Kbytes) */
-    int req_free = (strcmp (inbound, inbound_nonsecure) ?
-		    minfree : minfree_nonsecure);
+    unsigned long freespace, freespace2;
+    int req_free = ((secure_flag == P_SECURE) ? minfree : minfree_nonsecure);
 
-    if (req_free >= 0 &&
-	getfree (inbound) < (unsigned long)((size - sb.st_size + 1023) / 1024 + req_free))
+    freespace = getfree(buf);
+    freespace2 = getfree(inbound);
+    if (freespace > freespace2) freespace = freespace2;
+    if (sb.st_size > size)
     {
-      Log (1, "no enough free space in %s (%liK, req-d %liK)",
-	   inbound,
-	   (long) getfree (inbound),
-	   (long) (size - sb.st_size + 1023) / 1024 + req_free);
+      Log (1, "Partial size %lu > %lu (file size), delete partial", 
+           (unsigned long) sb.st_size, (unsigned long) size);
+      fclose (f);
+      if (trunc (buf) && delete (buf)) return 0;
+      goto fopen_again;
+    }
+    if (req_free >= 0 &&
+	freespace < (unsigned long)(size - sb.st_size + 1023) / 1024 + req_free)
+    {
+      Log (1, "no enough free space in %s (%luK, req-d %luK)",
+	   (freespace == freespace2) ? inbound : temp_inbound,
+	   freespace,
+	   (unsigned long) (size - sb.st_size + 1023) / 1024 + req_free);
       fclose (f);
       return 0;
     }
@@ -291,6 +361,7 @@ int inb_done (char *netname, off_t size, time_t time,
 {
   char tmp_name[MAXPATHLEN + 1];
   char *s, *u;
+  int  unlinked = 0, i;
 
   *real_name = 0;
 
@@ -303,51 +374,74 @@ int inb_done (char *netname, off_t size, time_t time,
   strnzcpy (real_name, inbound, MAXPATHLEN);
   strnzcat (real_name, PATH_SEPARATOR, MAXPATHLEN);
   s = real_name + strlen (real_name);
-  strnzcat (real_name, u = strdequote (netname), MAXPATHLEN);
+  strnzcat (real_name, u = makeinboundcase (strdequote (netname)), MAXPATHLEN);
   free (u);
   strwipe (s);
-  s = real_name + strlen (real_name) - 1;
 
-  /* gul: for *.pkt and *.?ic (tic, zic etc.) change name but not extension */
-  /* ditto for arcmail -- mff */
-  if (ispkt (netname) || istic (netname) || isarcmail (netname))
-    s -= 4;
-
-  if (touch (tmp_name, time) != 0)
-    Log (1, "touch %s: %s", tmp_name, strerror (errno));
-
-  while (1)
+  if (mask_test(netname, overwrite) && !ispkt(netname) && !isarcmail(netname))
   {
-    if (!RENAME (tmp_name, real_name))
+    for(i=0; ; i++)
     {
-      Log (2, "%s -> %s", netname, real_name);
-      break;
-    }
-    else
-    {
-      if (errno != EEXIST && errno != EACCES && errno != EAGAIN)
+      unlinked |= (unlink(real_name) == 0);
+      if (!RENAME (tmp_name, real_name))
+      {
+        Log (1, "%s -> %s%s", netname, real_name, unlinked?" (overwrited)":"");
+	break;
+      }
+      if ((errno != EEXIST && errno != EACCES && errno != EAGAIN) || i==10)
       {
         Log (1, "cannot rename %s to it's realname: %s! (data stored in %s)",
              netname, strerror (errno), tmp_name);
         *real_name = 0;
         return 0;
       }
-      Log (2, "error renaming `%s' to `%s': %s",
-	   netname, real_name, strerror (errno));
     }
+  } else
+  {
 
-    if (isalpha (*s) && toupper (*s) != 'Z')
-      ++*s;
-    else if (isdigit (*s) && toupper (*s) != '9')
-      ++*s;
-    else if (*s == '9')
-      *s = 'a';
-    else if (*--s == '.' || *s == '\\' || *s == '/')
+    s = real_name + strlen (real_name) - 1;
+
+    /* gul: for *.pkt and *.?ic (tic, zic etc.) change name but not extension */
+    /* ditto for arcmail -- mff */
+    if (ispkt (netname) || istic (netname) || isarcmail (netname) || isreq (netname))
+      s -= 4;
+
+    if (touch (tmp_name, time) != 0)
+      Log (1, "touch %s: %s", tmp_name, strerror (errno));
+
+    while (1)
     {
-      Log (1, "cannot rename %s to it's realname! (data stored in %s)",
-	   netname, tmp_name);
-      *real_name = 0;
-      return 0;
+      if (!RENAME (tmp_name, real_name))
+      {
+        Log (2, "%s -> %s", netname, real_name);
+        break;
+      }
+      else
+      {
+        if (errno != EEXIST && errno != EACCES && errno != EAGAIN)
+        {
+          Log (1, "cannot rename %s to it's realname: %s! (data stored in %s)",
+               netname, strerror (errno), tmp_name);
+          *real_name = 0;
+          return 0;
+        }
+        Log (2, "error renaming `%s' to `%s': %s",
+	     netname, real_name, strerror (errno));
+      }
+
+      if (isalpha (*s) && toupper (*s) != 'Z')
+        ++*s;
+      else if (isdigit (*s) && toupper (*s) != '9')
+        ++*s;
+      else if (*s == '9')
+        *s = 'a';
+      else if (*--s == '.' || *s == '\\' || *s == '/')
+      {
+        Log (1, "cannot rename %s to it's realname! (data stored in %s)",
+	     netname, tmp_name);
+        *real_name = 0;
+        return 0;
+      }
     }
   }
 
@@ -374,6 +468,6 @@ int inb_test (char *filename, off_t size, time_t t,
   free (u);
   strwipe (s);
 
-  return stat (fp, &sb) == 0 &&  sb.st_size == size &&
+  return stat (fp, &sb) == 0 && sb.st_size == size &&
     (sb.st_mtime & ~1) == (t & ~1);
 }
