@@ -12,9 +12,87 @@
  */
 
 /*
- * $Id: server.c,v 2.0 2001/01/10 12:12:39 gul Exp $
+ * $Id: server.c,v 2.23.2.1 2003/08/24 13:28:06 stream Exp $
  *
  * $Log: server.c,v $
+ * Revision 2.23.2.1  2003/08/24 13:28:06  stream
+ * Socket wasn't closed if branch() failed
+ *
+ * Revision 2.23  2003/05/23 18:10:57  stas
+ * Do not report errors when threads exits by exitfunc
+ *
+ * Revision 2.22  2003/05/04 08:45:30  gul
+ * Lock semaphores more safely for resolve and IP-addr print
+ *
+ * Revision 2.21  2003/04/28 07:30:17  gul
+ * Bugfix: Log() changes TCPERRNO
+ *
+ * Revision 2.20  2003/03/31 19:53:08  gul
+ * Close socket before exit
+ *
+ * Revision 2.19  2003/03/26 13:53:28  gul
+ * Fix OS/2 compilation
+ *
+ * Revision 2.18  2003/03/11 11:42:23  gul
+ * Use event semaphores for exit threads
+ *
+ * Revision 2.17  2003/03/11 00:04:26  gul
+ * Use patches for compile under MSDOS by MSC 6.0 with IBMTCPIP
+ *
+ * Revision 2.16  2003/03/10 17:32:37  gul
+ * Use socklen_t
+ *
+ * Revision 2.15  2003/03/10 12:16:54  gul
+ * Use HAVE_DOS_H macro
+ *
+ * Revision 2.14  2003/03/10 10:39:23  gul
+ * New include file common.h
+ *
+ * Revision 2.13  2003/03/10 08:38:07  gul
+ * Make n_servers/n_clients changes thread-safe
+ *
+ * Revision 2.12  2003/03/05 19:47:11  gul
+ * Fix compilation warning
+ *
+ * Revision 2.11  2003/03/05 13:21:51  gul
+ * Fix warnings
+ *
+ * Revision 2.10  2003/03/03 23:41:20  gul
+ * Try to resolve problem with active threads while exitproc running
+ *
+ * Revision 2.9  2003/03/01 20:16:27  gul
+ * OS/2 IBM C support
+ *
+ * Revision 2.8  2003/03/01 18:16:04  gul
+ * Use HAVE_SYS_TIME_H macro
+ *
+ * Revision 2.7  2003/03/01 15:55:02  gul
+ * Current outgoing address is now attibute of session, but not node
+ *
+ * Revision 2.6  2003/02/28 20:39:08  gul
+ * Code cleanup:
+ * change "()" to "(void)" in function declarations;
+ * change C++-style comments to C-style
+ *
+ * Revision 2.5  2003/02/22 12:12:34  gul
+ * Cleanup sources
+ *
+ * Revision 2.4  2002/11/12 16:55:58  gul
+ * Run as service under win9x
+ *
+ * Revision 2.3  2001/09/14 07:23:06  gul
+ * bindaddr bugfix, did not work on freebsd
+ *
+ * Revision 2.2  2001/08/24 13:23:28  da
+ * binkd/binkd.c
+ * binkd/readcfg.c
+ * binkd/readcfg.h
+ * binkd/server.c
+ * binkd/nt/service.c
+ *
+ * Revision 2.1  2001/05/23 16:48:04  gul
+ * msvc warnings fixed
+ *
  * Revision 2.0  2001/01/10 12:12:39  gul
  * Binkd is under CVS again
  *
@@ -39,7 +117,7 @@
 
 #include <sys/types.h>
 #include <time.h>
-#if !defined(VISUALCPP) && !defined(DOS)
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 #include <sys/stat.h>
@@ -53,32 +131,26 @@
 #include <signal.h>
 #include <setjmp.h>
 #elif defined(HAVE_THREADS)
-#ifndef __IBMC__
+#ifdef HAVE_DOS_H
 #include <dos.h>
 #endif
 #include <process.h>
-#else
+#elif !defined(DOS)
 #error Must define either HAVE_FORK or HAVE_THREADS!
 #endif
 
 #include "Config.h"
+#include "sys.h"
 #include "iphdr.h"
+#include "common.h"
 #include "iptools.h"
 #include "tools.h"
 #include "readcfg.h"
 #include "protocol.h"
 #include "server.h"
-#include "sys.h"
 #include "assert.h"
 #include "setpttl.h"
-
-#ifdef HAVE_THREADS
 #include "sem.h"
-extern MUTEXSEM hostsem;
-#ifdef OS2
-extern void rel_grow_handles(int nh);
-#endif
-#endif
 
 int n_servers = 0;
 int ext_rand = 0;
@@ -93,32 +165,33 @@ static void chld (int signo)
 
 #endif
 
-SOCKET sockfd = -1;
-extern int checkcfg_flag;	       /* exit(3) on config change */
+SOCKET sockfd = INVALID_SOCKET;
 
 void serv (void *arg)
 {
   int h = *(int *) arg;
-  extern int pidcmgr;
 
-  pidcmgr = 0;
 #ifdef HAVE_FORK
+  pidcmgr = 0;
   soclose(sockfd);
+  sockfd = INVALID_SOCKET;
 #endif
-  protocol (h, 0);
+  protocol (h, 0, NULL);
   Log (5, "downing server...");
+  del_socket(h);
   soclose (h);
   free (arg);
-#ifdef HAVE_THREADS
-#ifdef OS2
   rel_grow_handles (-6);
-#endif
-  --n_servers;
+#ifdef HAVE_THREADS
+  threadsafe(--n_servers);
+  PostSem(&eothread);
   _endthread();
+#elif defined(DOS)
+  --n_servers;
 #endif
 }
 
-int checkcfg ()
+int checkcfg (void)
 {
   struct stat sb;
   struct conflist_type *pc;
@@ -134,14 +207,17 @@ int checkcfg ()
     {
       pc->mtime = (unsigned long)sb.st_mtime;
     }
-    else if (pc->mtime != sb.st_mtime)
+    else if ((time_t)pc->mtime != sb.st_mtime)
     {
-      soclose (sockfd);
 #if defined(HAVE_FORK)
       Log (2, "%s changed! Restart binkd...", pc->path);
       longjmp(jb, 1);
+#else
+#if defined(BINKDW9X)
+      Log (2, "%s changed! Restart binkd...", pc->path);
 #else
       Log (2, "%s changed! exit(3)...", pc->path);
+#endif
       checkcfg_flag=2;
 #endif
       exit (3);
@@ -154,11 +230,12 @@ void servmgr (void *arg)
 {
   SOCKET new_sockfd;
   int pid;
-  int client_addr_len;
+  socklen_t client_addr_len;
   struct sockaddr_in serv_addr, client_addr;
   int opt = 1;
+  int save_errno;
 
-  srand((unsigned int)time(0));
+  srand(time(0));
   setproctitle ("server manager");
   Log (4, "servmgr started");
 
@@ -176,8 +253,12 @@ void servmgr (void *arg)
 		  (char *) &opt, sizeof opt) == SOCKET_ERROR)
     Log (1, "setsockopt (SO_REUSEADDR): %s", TCPERR ());
 
+  memset(&serv_addr, 0, sizeof serv_addr);
   serv_addr.sin_family = AF_INET;
-  serv_addr.sin_addr.s_addr = htonl (INADDR_ANY);
+  if (bindaddr[0])
+    serv_addr.sin_addr.s_addr = inet_addr (bindaddr);
+  else
+    serv_addr.sin_addr.s_addr = htonl (INADDR_ANY);
   serv_addr.sin_port = htons ((unsigned short) iport);
 
   if (bind (sockfd, (struct sockaddr *) & serv_addr, sizeof (serv_addr)) != 0)
@@ -204,6 +285,7 @@ void servmgr (void *arg)
       case -1:
         if (TCPERRNO == EINTR)
           continue;
+	save_errno = TCPERRNO;
 	Log (1, "select: %s", TCPERR ());
         goto accepterr;
     }
@@ -216,18 +298,20 @@ void servmgr (void *arg)
     if ((new_sockfd = accept (sockfd, (struct sockaddr *) & client_addr,
 			      &client_addr_len)) == INVALID_SOCKET)
     {
-      if (TCPERRNO != EINVAL && TCPERRNO != EINTR)
+      save_errno = TCPERRNO;
+      if (save_errno != EINVAL && save_errno != EINTR)
       {
-	Log (1, "accept: %s", TCPERR ());
+	if (!binkd_exit)
+	  Log (1, "accept: %s", TCPERR ());
 #ifdef UNIX
-	if (TCPERRNO == ECONNRESET ||
-	    TCPERRNO == ETIMEDOUT ||
-	    TCPERRNO == EHOSTUNREACH)
+	if (save_errno == ECONNRESET ||
+	    save_errno == ETIMEDOUT ||
+	    save_errno == EHOSTUNREACH)
 	   continue;
 #endif
 accepterr:
 #ifdef OS2
-	if (TCPERRNO == ENOTSOCK)
+	if (save_errno == ENOTSOCK)
 	{ /* os/2 ugly hack */
 	  if (config_list)
 	  { config_list->mtime--;
@@ -240,29 +324,25 @@ accepterr:
     }
     else
     { char host[MAXHOSTNAMELEN + 1];
-#if defined(HAVE_THREADS) && defined(OS2)
-      rel_grow_handles (6);
-#endif
 
+      add_socket(new_sockfd);
+      rel_grow_handles (6);
       ext_rand=rand();
-#ifdef HAVE_THREADS
-      LockSem(&hostsem);
-#endif
-      Log (3, "incoming from %s (%s)",
-	   get_hostname(&client_addr, host, sizeof(host)),
+      get_hostname(&client_addr, host, sizeof(host));
+      lockhostsem();
+      Log (3, "incoming from %s (%s)", host,
 	   inet_ntoa (client_addr.sin_addr));
-#ifdef HAVE_THREADS
-      ReleaseSem(&hostsem);
-#endif
+      releasehostsem();
 
       /* Creating a new process for the incoming connection */
-      ++n_servers;
+      threadsafe(++n_servers);
       if ((pid = branch (serv, (void *) &new_sockfd, sizeof (new_sockfd))) < 0)
       {
-#if defined(HAVE_THREADS) && defined(OS2)
+        del_socket(new_sockfd);
+        soclose(new_sockfd);
         rel_grow_handles (-6);
-#endif
-	--n_servers;
+	threadsafe(--n_servers);
+	PostSem(&eothread);
 	Log (1, "cannot branch out");
         sleep(1);
       }
