@@ -12,9 +12,80 @@
  */
 
 /*
- * $Id: tools.c,v 2.0 2001/01/10 12:12:39 gul Exp $
+ * $Id: tools.c,v 2.22.2.1 2003/06/30 22:46:03 hbrew Exp $
  *
  * $Log: tools.c,v $
+ * Revision 2.22.2.1  2003/06/30 22:46:03  hbrew
+ * Print only binkd name (without path) in error messages
+ *
+ * Revision 2.22  2003/04/02 13:12:57  gul
+ * Try to use workaround for buggy windows time functions (timezone)
+ *
+ * Revision 2.21  2003/03/31 22:47:22  gul
+ * remove workaround for msvc localtime() bug
+ *
+ * Revision 2.20  2003/03/31 21:48:59  gul
+ * Avoid infinite recursion
+ *
+ * Revision 2.19  2003/03/31 21:27:12  gul
+ * Avoid infinite recursion
+ *
+ * Revision 2.18  2003/03/31 20:28:24  gul
+ * safe_localtime() and safe_gmtime() functions
+ *
+ * Revision 2.17  2003/03/31 19:35:16  gul
+ * Clean semaphores usage
+ *
+ * Revision 2.16  2003/03/31 18:22:12  gul
+ * Use snprintf() instead of sprintf()
+ *
+ * Revision 2.15  2003/03/10 10:57:45  gul
+ * Extern declarations moved to header files
+ *
+ * Revision 2.14  2003/03/05 13:21:51  gul
+ * Fix warnings
+ *
+ * Revision 2.13  2003/02/28 20:39:08  gul
+ * Code cleanup:
+ * change "()" to "(void)" in function declarations;
+ * change C++-style comments to C-style
+ *
+ * Revision 2.12  2002/11/14 09:46:59  gul
+ * Minor BINKDW9X fix
+ *
+ * Revision 2.11  2002/11/12 16:55:58  gul
+ * Run as service under win9x
+ *
+ * Revision 2.10  2002/05/06 19:25:40  gul
+ * new keyword inboundCase in config
+ *
+ * Revision 2.9  2002/03/20 15:31:19  gul
+ * ftrans bugfix
+ *
+ * Revision 2.8  2002/03/20 14:32:14  gul
+ * Bugfix in ftrans
+ *
+ * Revision 2.7  2002/03/07 14:29:52  gul
+ * print PID as unsigned to bsy/csy
+ *
+ * Revision 2.6  2002/02/25 21:33:56  gul
+ * Dequote \hh in filenames as \xhh; both \20 and \x20 are space now (FSP-1011)
+ *
+ * Revision 2.5  2001/11/08 14:04:12  gul
+ * bugfix
+ *
+ * Revision 2.4  2001/11/07 17:01:12  gul
+ * Check size of buffer in strnzcpy()
+ *
+ * Revision 2.3  2001/10/23 08:33:44  gul
+ * Change filename (not ext) in incoming *.req if already exists
+ *
+ * Revision 2.2  2001/09/30 13:49:59  gul
+ * Do not put log to socket if run via inetd
+ *
+ * Revision 2.1  2001/09/24 10:31:39  gul
+ * Build under mingw32
+ *
  * Revision 2.0  2001/01/10 12:12:39  gul
  * Binkd is under CVS again
  *
@@ -88,15 +159,12 @@
 #include "assert.h"
 #include "Config.h"
 #include "sys.h"
+#include "common.h"
 #include "tools.h"
 #include "readdir.h"		       /* for [sys/]utime.h */
 #include "readcfg.h"
 #include "sem.h"
 
-#ifdef WIN32
-extern int isService;
-#endif
-
 /*
  * Lowercase the string
  */
@@ -105,7 +173,19 @@ char *strlower (char *s)
   int i;
 
   for (i = 0; s[i]; ++i)
-    s[i] = (char) tolower (s[i]);
+    s[i] = tolower (s[i]);
+  return s;
+}
+
+/*
+ * Uppercase the string
+ */
+char *strupper (char *s)
+{
+  int i;
+
+  for (i = 0; s[i]; ++i)
+    s[i] = toupper (s[i]);
   return s;
 }
 
@@ -166,14 +246,14 @@ int mkpath (char *s)
   return mkpath0 (path);
 }
 
-unsigned long rnd ()
+unsigned long rnd (void)
 {
   static int i;
 
   if (!i)
   {
     i = 1;
-    srand ((unsigned int)time (0));
+    srand (time (0));
   }
   return (time (0) + rand ()) & 0xFFFFFFFFul;
 }
@@ -194,13 +274,17 @@ int create_empty_sem_file (char *name)
 int create_sem_file (char *name)
 {
   int h, i;
-  char buf[10];
+  char buf[16];
 
   if ((h = open (name, O_RDWR | O_CREAT | O_EXCL, 0666)) == -1)
   { Log (5, "Can't create %s: %s", name, strerror(errno));
     return 0;
   }
-  sprintf (buf, "%i\n", (int) getpid ());
+#ifdef HAVE_SNPRINTF
+  snprintf (buf, sizeof (buf), "%u\n", (int) getpid ());
+#else
+  sprintf (buf, "%u\n", (int) getpid ());
+#endif
   if ((i = write(h, buf, strlen(buf))) != (int)strlen(buf))
   { if (i == -1)
       Log (2, "Can't write to %s (handle %d): %s", name, h, strerror(errno));
@@ -216,39 +300,128 @@ int create_sem_file (char *name)
 #include <malloc.h>		       /* for _heapchk() */
 #endif
 
+struct tm *safe_gmtime(time_t *t, struct tm *tm)
+{
+  threadsafe(memcpy(tm, gmtime(t), sizeof(*tm)));
+  return tm;
+}
+
+#ifdef WIN32
+#include <windows.h>
+
+static void stime_to_tm(const SYSTEMTIME *stime, struct tm *tm)
+{
+  tm->tm_year = stime->wYear-1900;
+  tm->tm_mon  = stime->wMonth-1;
+  tm->tm_mday = stime->wDay;
+  tm->tm_wday = stime->wDayOfWeek;
+  tm->tm_hour = stime->wHour;
+  tm->tm_min  = stime->wMinute;
+  tm->tm_sec  = stime->wSecond;
+}
+
+static void tm_to_stime(const struct tm *tm, SYSTEMTIME *stime)
+{
+  stime->wYear         = tm->tm_year+1900;
+  stime->wMonth        = tm->tm_mon+1;
+  stime->wDay          = tm->tm_mday;
+  stime->wDayOfWeek    = tm->tm_wday;
+  stime->wHour         = tm->tm_hour;
+  stime->wMinute       = tm->tm_min;
+  stime->wSecond       = tm->tm_sec;
+  stime->wMilliseconds = 0;
+}
+
+static int safe_cmptime(const struct tm *tm1, const struct tm *tm2)
+{
+  if (tm1->tm_year > tm2->tm_year) return  1;
+  if (tm1->tm_year < tm2->tm_year) return -1;
+  if (tm1->tm_mon  > tm2->tm_mon ) return  1;
+  if (tm1->tm_mon  < tm2->tm_mon ) return -1;
+  if (tm1->tm_mday > tm2->tm_mday) return  1;
+  if (tm1->tm_mday < tm2->tm_mday) return -1;
+  if (tm1->tm_hour > tm2->tm_hour) return  1;
+  if (tm1->tm_hour < tm2->tm_hour) return -1;
+  return 0;
+}
+
+time_t safe_time(void)
+{
+  /* gmtime(t) should be SystemTime */
+  SYSTEMTIME stime;
+  struct tm utctm, tm;
+  time_t t;
+  int i;
+
+  GetSystemTime(&stime);
+  stime_to_tm(&stime, &utctm);
+  t = time(NULL);
+  t -= (t-utctm.tm_sec)%60;
+  for (i=0; i<24; i++) {
+    safe_gmtime(&t, &tm);
+    if (tm.tm_hour == utctm.tm_hour) return t;
+    t += safe_cmptime(&tm, &utctm) > 0 ? -3600 : 3600;
+  }
+  return t;
+}
+
+struct tm *safe_localtime(time_t *t, struct tm *tm)
+{
+  FILETIME utcftime, localftime;
+  SYSTEMTIME stime;
+
+  safe_gmtime(t, tm);
+  /* convert gmtime to localtime */
+  tm_to_stime(tm, &stime);
+  SystemTimeToFileTime(&stime, &utcftime);
+  FileTimeToLocalFileTime(&utcftime, &localftime);
+  FileTimeToSystemTime(&localftime, &stime);
+  stime_to_tm(&stime, tm);
+  return tm;
+}
+#else
+struct tm *safe_localtime(time_t *t, struct tm *tm)
+{
+  threadsafe(memcpy(tm, localtime(t), sizeof(*tm)));
+  return tm;
+}
+
+time_t safe_time(void)
+{
+  return time(NULL);
+}
+#endif
+
+#if defined(HAVE_THREADS) || defined(AMIGA)
+MUTEXSEM LSem = 0;
+#endif
+
 void Log (int lev, char *s,...)
 {
-#if defined(HAVE_THREADS) || defined(AMIGA)
-
-  static MUTEXSEM LSem;
-
-#endif
-#if defined (HAVE_VSYSLOG) && defined (HAVE_FACILITYNAMES)
-
-  extern int syslog_facility;
-
-#endif
-
   static int first_time = 1;
   char timebuf[60];
   time_t t;
-  struct tm *tm;
+  struct tm tm;
   va_list ap;
   static const char *marks = "!?+-";
-  char ch = (char)((0 <= lev && lev < (int) strlen (marks)) ? marks[lev] : ' ');
+  char ch = (0 <= lev && lev < (int) strlen (marks)) ? marks[lev] : ' ';
 
   if (first_time == 1)
   {
-    InitSem (&LSem);
     first_time = 2;
+    InitSem (&LSem);
   }
 
-  time (&t);
-  tm = localtime (&t);
+  t = safe_time();
+  safe_localtime (&t, &tm);
 
-  if (lev <= conlog)
+  if (lev <= conlog
+#if defined(UNIX) || defined(OS2) || defined(AMIGA)
+      && !inetd_flag
+#endif
+     )
   {
-    strftime (timebuf, sizeof (timebuf), "%H:%M", tm);
+    strftime (timebuf, sizeof (timebuf), "%H:%M", &tm);
     LockSem (&LSem);
     fprintf (stderr, "%30.30s\r%c %s [%i] ", " ", ch, timebuf, (int) PID ());
     va_start (ap, s);
@@ -278,7 +451,7 @@ void Log (int lev, char *s,...)
 	fputc ('\n', logfile);
 	first_time = 0;
       }
-      strftime (timebuf, sizeof (timebuf), "%d %b %H:%M:%S", tm);
+      strftime (timebuf, sizeof (timebuf), "%d %b %H:%M:%S", &tm);
       fprintf (logfile, "%c %s [%i] ", ch, timebuf, (int) PID ());
       va_start (ap, s);
       vfprintf (logfile, s, ap);
@@ -291,7 +464,11 @@ void Log (int lev, char *s,...)
     ReleaseSem (&LSem);
   }
 #ifdef WIN32
+#ifdef BINKDW9X
+  if(!lev)
+#else
   if((!lev)&&(isService))
+#endif
   {
     char tmp[256];
     va_start (ap, s);
@@ -416,13 +593,18 @@ char *strdequote (char *s)
 
   while (*s)
   {
+#define XD(x) (isdigit(x) ? ((x)-'0') : (tolower(x)-'a'+10))
     if (s[0] == '\\' && s[1] == 'x' && isxdigit (s[2]) && isxdigit (s[3]))
     {
-#define XD(x) (isdigit(x) ? ((x)-'0') : (tolower(x)-'a'+10))
-      r[i++] = (char) (XD (s[2]) * 16 + XD (s[3]));
+      r[i++] = XD (s[2]) * 16 + XD (s[3]);
       s += 4;
-#undef XD
     }
+    else if (s[0] == '\\' && isxdigit (s[1]) && isxdigit (s[2]))
+    {
+      r[i++] = XD (s[1]) * 16 + XD (s[2]);
+      s += 3;
+    }
+#undef XD
     else
       r[i++] = *(s++);
   }
@@ -440,7 +622,7 @@ char *strwipe (char *s)
   for (i = 0; s[i]; ++i)
     if ((iscntrl (s[i]) || s[i] == '\\' || s[i] == '/' || s[i] == ':')
 #ifdef WIN32 /* ungly hack */
-        && (s[i] & 0x80 == 0)
+        && ((s[i] & 0x80) == 0)
 #endif
        )
       s[i] = '_';
@@ -459,8 +641,9 @@ char *strnzcpy (char *dst, const char *src, size_t len)
 
 char *strnzcat (char *dst, const char *src, size_t len)
 {
-  int x = strlen (dst);
+  size_t x = strlen (dst);
 
+  if (len <= x) return dst;
   return strnzcpy (dst + x, src, len - x);
 }
 
@@ -513,26 +696,20 @@ int touch (char *file, time_t t)
 
   if ((r = stat(file, &st)) == 0)
   {
-    struct tm *tm;
-#ifdef __WATCOMC__
-    struct tm stm;
-    tm = &stm;
-    _localtime(&t, tm);
-#else
-    tm=localtime(&t);
-#endif
+    struct tm tm;
+    safe_localtime(&t, &tm);
     buf.fdateCreation.day=buf.fdateLastAccess.day=buf.fdateLastWrite.day=
-        tm->tm_mday;
+        tm.tm_mday;
     buf.fdateCreation.month=buf.fdateLastAccess.month=buf.fdateLastWrite.month=
-        tm->tm_mon+1;
+        tm.tm_mon+1;
     buf.fdateCreation.year=buf.fdateLastAccess.year=buf.fdateLastWrite.year=
-        tm->tm_year-80;
+        tm.tm_year-80;
     buf.ftimeCreation.twosecs=buf.ftimeLastAccess.twosecs=buf.ftimeLastWrite.twosecs=
-        tm->tm_sec/2;
+        tm.tm_sec/2;
     buf.ftimeCreation.minutes=buf.ftimeLastAccess.minutes=buf.ftimeLastWrite.minutes=
-        tm->tm_min;
+        tm.tm_min;
     buf.ftimeCreation.hours=buf.ftimeLastAccess.hours=buf.ftimeLastWrite.hours=
-        tm->tm_hour;
+        tm.tm_hour;
     buf.cbFile = buf.cbFileAlloc = st.st_size;
     buf.attrFile = FILE_ARCHIVED | FILE_NORMAL;
     r=DosSetPathInfo(file, FIL_STANDARD, &buf, sizeof(buf), 0);
@@ -550,41 +727,47 @@ int touch (char *file, time_t t)
  * Returned value must be free()'d. Ignores case.
  * size parameter can be used only if "s" was created by malloc()
  */
-char *ed (char *s, char *a, char *b, unsigned int *size)
+char *ed (char *s, char *a, char *b, size_t *size)
 {
-   int i, j;
-   unsigned int sz;
-   int len_a=a?strlen(a):0;
-   int len_b=b?strlen(b):0;
+   size_t i, j, sr, sz;
+   size_t len_a=a?strlen(a):0;
+   size_t len_b=b?strlen(b):0;
    char *r=s;
    
-   if((!len_a)||(!s)) return r;
-   if(!size) 
+   if ((!len_a) || (!s)) return r;
+   if (!size) 
    {
      sz=strlen(s)+1;
      r=xstrdup(s);
    }
    else sz=*size;
-   for(i=j=0;i<(int)strlen(r);i++)
+   sr=strlen(r);
+   for (i=j=0; i<sr; i++)
    {
-     if(tolower(r[i])!=tolower(a[j++]))
+     if (tolower(r[i])!=tolower(a[j]))
      {
+       i-=j;
        j=0;
        continue;
      }
-     if(a[j]) continue;
-     if(strlen(r)-len_a+len_b>sz)
+     j++;
+     if (a[j]) continue;
+     if (sr-len_a+len_b>=sz)
      {
-       if(len_b<64) sz+=64;
+       if (len_b<64) sz+=64;
        else sz+=len_b;
        r=xrealloc(r, sz);
      }
      i-=len_a-1;
-     memmove(r+i+len_b, r+i+len_a, strlen(r+i+len_a)+1);
-     if(len_b)
+     if (len_a!=len_b)
+       memmove(r+i+len_b, r+i+len_a, sr-i-len_a+1);
+     if (len_b)
        memcpy(r+i, b, len_b);
+     j=0;
+     i+=len_b-1;
+     sr+=len_b-len_a;
    }
-   if(size) *size=sz;
+   if (size) *size=sz;
    return r;
 }
 
@@ -627,7 +810,7 @@ int trunc (char *path)
 #include <sys/utsname.h>
 #endif
 
-char *get_os_string ()
+char *get_os_string (void)
 {
   static char os[80];
 
@@ -662,6 +845,11 @@ int ispkt (char *s)
   return pmatch ("*.[Pp][Kk][Tt]", s);
 }
 
+int isreq (char *s)
+{
+  return pmatch ("*.[Rr][Ee][Qq]", s);
+}
+
 int istic (char *s)
 {
   return pmatch ("*.?[Ii][Cc]", s);
@@ -727,3 +915,41 @@ char **mkargv (int argc, char **argv)
 
   return p;
 }
+
+/*
+ * Apply filename case style defined in inboundcase
+ */
+char *makeinboundcase (char *s)
+{
+  int i;
+  
+  switch (inboundcase) 
+  {
+      case INB_UPPER:
+	s = strupper(s);
+	Log (8, "uppercase filename");
+        break;
+      case INB_LOWER:
+	s = strlower(s);
+	Log (8, "lowercase filename");
+        break;
+      case INB_MIXED:
+        s[0] = toupper (s[0]);
+        for (i = 1; s[i]; ++i)
+          s[i] = isalnum(s[i-1]) ? tolower(s[i]) : toupper(s[i]);
+        Log (8, "mixing filename case");
+        break;
+      default:
+	Log (8, "nothing to do with filename case");
+        break;
+  }
+
+  return s;
+}
+
+/* Extract filename from path */
+char *extract_filename(char *s)
+{
+  char *tmp = max(max(strrchr(s, '\\'), strrchr(s, '/')), strrchr(s, ':'));
+  return tmp? tmp+1: s;
+}
